module Rubtools
  module Tools
    class Svn < Rubtools::Recipe
      register_methods %w(co_all up_all print_repos)

      def initialize
        @svn = which "svn"
        
        @svn_config = config.svn
        raise "Add the key 'svn' in the config file" unless @svn_config

        @install_dir = @svn_config.install_dir
        raise "Add the key install_dir: /path/to/folder in the config file" unless @install_dir
        raise "Subversion isn't installed" unless @svn
        raise "Folder doesn't exists: #{@install_dir}" unless File.exists? @install_dir
      end

      ## Print all repositories
      def print_repos
        for repo in config.svn.repositories
          success "#{repo.name}\t#{repo.url}"
        end
      end

      ## Cloning all repositories located into rubtools.yml
      #
      def co_all
        for repo in config.svn.repositories
          co repo
        end
      end

      ## Pull all repositories located into rubtools.yml
      #
      def up_all
        for repo in config.svn.repositories
          up repo
        end
      end

      private
      ## Cloning a specific repository
      #
      def co repo
          repo_path = File.join @install_dir, repo.name

          unless File.exists? repo_path
            success "Checkouting #{repo.name} from #{repo.url}"
            exec("#{@svn} checkout #{repo.url} #{repo_path}") do |status|
              success "> done" if status.success?
            end
          else
            success "Repository #{repo.name} already checkout"
          end
      end


      ## Pulling a specific repository
      #
      def up repo
          repo_path = File.join @install_dir, repo.name

          if File.exists? repo_path
            success "Updating #{repo.name} repository"
            exec "#{@svn} update #{repo.url}" do |status|
              success "> done" if status.success?
            end
          else
            error "Repository #{repo.name} doensn't exists"
            co repo
          end
      end
    end
  end
end

